@@ -1,5 +1,6 @@
 from dataclasses import dataclass
 from typing import Dict, TypeVar, Union, NamedTuple, Optional, Any, Callable, List, Iterable, Tuple, Mapping, Sequence
+from abc import ABC, abstractmethod
 import functools
 import sys
 import operator
@@ -46,10 +47,11 @@ def outline_union_strip_optional(a: Optional[Strokes], b: Optional[Strokes])->St
 	assert b
 	return outline_union(a, b)
 
-class Dictionary:
+class Dictionary(ABC):
 	def __init__(self, stroke_type: type)->None:
 		self.stroke_type=stroke_type
 
+	@abstractmethod
 	def lookup(self, strokes: Strokes)->Any:
 		"""
 		Lookup a stroke in the dictionary.
@@ -58,6 +60,9 @@ class Dictionary:
 		"""
 		raise NotImplementedError
 
+	def __getitem__(self, strokes: Strokes)->Any:
+		return self.lookup(strokes)
+
 	def lookup_str(self, strokes: str)->LookupResult:
 		"""
 		Lookup a stroke in the dictionary.
@@ -84,12 +89,17 @@ class Dictionary:
 		assert result is None or isinstance(result, str), result
 		return result
 	
+	@abstractmethod
 	def items(self)->Iterable[Tuple[Strokes, Any]]:
 		"""
 		Return all items in the dictionary.
 		"""
 		raise NotImplementedError
 
+	def keys(self)->Iterable[Strokes]:
+		for key, value in self.items():
+			yield key
+
 	def items_str(self)->Iterable[Tuple[str, str]]:
 		for key, value in self.items():
 			assert isinstance(value, str), value
@@ -151,6 +161,15 @@ class Dictionary:
 	def named(self, name: str)->"NamedDictionary":
 		return NamedDictionary(self.stroke_type, self, name)
 
+	def __str__(self)->str:
+		"""
+		Pretty-print this dictionary.
+		"""
+		return self.__class__.__name__ + "(" + str(dict(self)) + ")"
+
+	def __repr__(self)->str:
+		return str(self)
+
 	outline_length: Optional[int]
 	"""
 	The length of the outlines in the dictionary, or None if the information is unknown.
@@ -175,7 +194,8 @@ def to_stroke(stroke_type: type, stroke: InputStrokeType)->BaseStroke:
 
 def to_strokes(stroke_type: type, strokes: InputStrokesType)->Strokes:
 	if isinstance(strokes, str):
-		return (stroke_type(strokes),)
+		# plover.steno.STROKE_DELIMITER
+		return tuple(stroke_type(stroke) for stroke in strokes.split("/"))
 	elif isinstance(strokes, BaseStroke):
 		return (strokes,)
 	elif isinstance(strokes, Iterable): # must handle str case before this one
@@ -486,7 +506,7 @@ def stroke(stroke_type: type, strokes: str)->Dictionary:
 	"""
 	return SingleDictionary(stroke_type, {strokes: ""})
 
-def translation(stroke_type: type, translation: str)->Dictionary:
+def translation(stroke_type: type, translation: Any)->Dictionary:
 	"""
 	Return a dictionary that has <translation> as the translation and nothing as the stroke.
 	Useful as part of a dictionary.
@@ -505,22 +525,26 @@ SubsetDictionary_=SubsetDictionary
 
 @dataclass
 class Context:
-	# first argument is self. Mypy currently doesn't type-check correctly
-	Stroke            : Callable[[Any,    InputStrokeType], BaseStroke]
-	stroke_type       : Callable[[Any,    InputStrokeType], BaseStroke]
-	SingleDictionary  : Callable[[Any,    InputStrokeType], SingleDictionary_]
-	s                 : Callable[[Any,    InputStrokeType], SingleDictionary_]
+	# In older mypy versions this would not type check correctly.
+	# https://github.com/python/mypy/issues/708
 
-	stroke            : Callable[[Any,    str], Dictionary]  # consist of that single stroke. See example file for details
+	# Stroke class provided by `plover_stroke` library.
+	Stroke            : Callable[[InputStrokeType], BaseStroke]
+	stroke_type       : Callable[[InputStrokeType], BaseStroke]
 
-	translation       : Callable[[Any,    Any], Dictionary]  # consist of a "null stroke" that translates to the translation. Input type is most frequently str
+	SingleDictionary  : Callable[[Union[Iterable[InputStrokesType], Dict[InputStrokesType, Any]]], SingleDictionary_]
+	s                 : Callable[[Union[Iterable[InputStrokesType], Dict[InputStrokesType, Any]]], SingleDictionary_]
 
-	filtered          : Callable[[Any,    Any], FilteredDictionary_]
-	FilteredDictionary: Callable[[Any,    Any], FilteredDictionary_]
+	stroke            : Callable[[str], Dictionary]  # consist of that single stroke. See example file for details
 
-	subsets           : Callable[[Any,    Any], SubsetDictionary_]
-	subsetd           : Callable[[Any,    Any], SubsetDictionary_]
-	SubsetDictionary  : Callable[[Any,    Any], SubsetDictionary_]
+	translation       : Callable[[Any], Dictionary]  # consist of a "null stroke" that translates to the translation. Input type is most frequently str
+
+	filtered          : Callable[[Any], FilteredDictionary_]
+	FilteredDictionary: Callable[[Any], FilteredDictionary_]
+
+	subsets           : Callable[[Any], SubsetDictionary_]
+	subsetd           : Callable[[Any], SubsetDictionary_]
+	SubsetDictionary  : Callable[[Any], SubsetDictionary_]
 
 
 
